-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2009, 2010, 2011 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

newTalent{
	name = "Arcane Spread",
	type = {"spell/arcane-power", 1},
	require = spells_req_high1,
	points = 5,
	mana = 12,
	cooldown = 3,
	tactical = { ATTACKAREA = 1, ATTACK = 1 },
	range = 10,
	radius = 1,
	proj_speed = 6,
	requires_target = true,
	target = function(self, t)
		return {type="ball", range=self:getTalentRange(t), radius=self:getTalentRadius(t), talent=t}
	end,
	getDamage = function(self, t) return self:combatTalentSpellDamage(t, 35, 200) end,
	action = function(self, t)
		local tg = self:getTalentTarget(t)
		local x, y = self:getTarget(tg)
		if not x or not y then return nil end
		local grids = self:project(tg, x, y, function(px, py)
			local actor = game.level.map(px, py, Map.ACTOR)
			if actor ~= self then
				local tg2 = {type="bolt", range=self:getTalentRange(t), talent=t}
				self:projectile(tg2, px, py, DamageType.ARCANE, self:spellCrit(t.getDamage(self, t))) --, {type="freeze"}
				game.level.map:particleEmitter(px, py, 1, "manathrust")
			end
		end)

		return true
	end,
	info = function(self, t)
		local damage = t.getDamage(self, t)
		return ([[Invoke mana shards at the selected area. Each shard travels slowly and does %0.2f arcane damage on impact.
		This spell will never target the caster.
		The damage will increase with the Magic stat]]):
		format(damDesc(self, DamageType.ARCANE, damage))
	end,
}

newTalent{
	short_name = "ARCANE_TRANSFER",
	name = "Arcane Transfer",
	type = {"spell/arcane-power", 2},
	require = spells_req_high2,
	points = 5,
	mode = "sustained",
	cooldown = 1,
	sustain_mana = 70,
	transfer  = function(self,t,src)
	local effs = {}
           if(rng.percent(self:getTalentLevel(t)*2)) and src ~= self then--rng.percent(self:getTalentLevel(t)*2)
		   
		   --clean self
		   for eff_id, p in pairs(self.tmp) do
				local e = self.tempeffect_def[eff_id]
				if e.status == "detrimental" and e.type == "magical" then --
					effs[#effs+1] = { eff_id,p,e.status}
					--self:removeEffect(eff_id)
				end
            end
             --take enemy
            for eff_id, p in pairs(src.tmp) do
                local e = src.tempeffect_def[eff_id]
                if e.status == "beneficial"  and e.type == "magical" then --
                    effs[#effs+1] = { eff_id,p,e.status}
					--src:removeEffect(eff_id)
                end
            end
            local nb = 1+math.floor(self:getTalentLevel(t))
            for i = 1, nb do
                print("intered for ",i)
                if #effs == 0 then break end
                    local eff = rng.tableRemove(effs)
                    
                    if eff[3] == "beneficial" then
                        --transfer to player
						src:removeEffect(eff[1])
                        self:setEffect(eff[1],eff[2].dur,eff[2])
                    elseif  eff[3] == "detrimental" and not self:attr("encased_in_ice") then
						--give to enemy
						self:removeEffect(eff[1])
                        src:setEffect(eff[1],eff[2].dur,eff[2])
                    end
                 end
            end
            return true
            end,
	activate = function(self, t)
		return {
			particle = self:addParticles(Particles.new("disruption_shield", 1)),
		}
	end,
	deactivate = function(self, t, p)
		self:removeParticles(p.particle)
		return true
	end,
	
	info = function(self, t)
		return ([[Whenever you deal arcane damage there is a %d chance that up to %d magical effect will be transfered.  Detrimental ones will go to the target and positive ones will go to you]]):
		format(self:getTalentLevel(2)*2,1+math.floor(self:getTalentLevel(t)))
	end,
}

newTalent{
	name = "Mana Drain",
	type = {"spell/arcane-power", 3},
	require = spells_req_high3,
	points = 5,
	random_ego = "utility",
	mana = 30,
	cooldown = 40,
	range = 7,
	tactical = { MANA = 3 },
	action = function(self, t)
		local tx, ty = self:getTarget{type="hit", range=self:getTalentRange(t)}
			local target
			if tx and ty and game.level.map(tx, ty, Map.ACTOR) then
				target = game.level.map(tx, ty, Map.ACTOR)
			end
			if target then
				target:setEffect(target.EFF_MANADRAIN,5,{src=self,power = self:combatTalentSpellDamage(t, 25, 100)})
			end
		return true
	end,
	info = function(self, t)
		return ([[Cause mana to slowly rip the target apart along with those around it.  It will deal %d arcane damage over 5 turns and will deal 1/10 of that to those around the target.  Damage will be increased if the target has mana (will drain the target of 30 mana a turn]]):
		format(self:combatTalentSpellDamage(t, 25, 100))
	end,
}

newTalent{
	short_name = "ARCANE_CONVERSION",
	name = "Arcane Conversion",
	type = {"spell/arcane-power",4},
	require = spells_req_high4, 
	points = 5,
	mode = "sustained",
	cooldown = 1,
	sustain_mana = 150,
	getPercentConverted = function(self,t) return self:combatTalentSpellDamage(t, 20, 75) end,
	activate = function(self, t)
		game:playSoundNear(self, "talents/arcane")
		return {
			particle = self:addParticles(Particles.new("disruption_shield", 1)),
		}
	end,
	deactivate = function(self, t, p)
		self:removeParticles(p.particle)
		return true
	end,
	info = function(self, t)
	local convertFactor = t.getPercentConverted(self,t)
		return ([[Convert %d of all damage dealt into arcane damage]]):
		format(convertFactor)
	end,
}
